# -*- coding: utf-8 -*-
"""
    converter
    ~~~~~~~~~~~~~~~~~~~

    converter
    SV <-> VCF Record

    @Copyright: (c) 2018-04 by Lingxi Chen (chanlingxi@gmail.com). 
    @License: LICENSE_NAME, see LICENSE for more details.
"""

import vcf_aux
import sv



if __name__ == "__main__":
    print(vcf2sv(1,3))
    print(vcf_aux.MyRecord())
